@@ -47,12 +47,14 @@ static const char *cops_prefix[] = { "+COPS:", NULL };
 static const char *csq_prefix[] = { "+CSQ:", NULL };
 static const char *cind_prefix[] = { "+CIND:", NULL };
 
-#define SIGNAL_STRENGTH_IND 2
-
 struct netreg_data {
 	GAtChat *chat;
 	char mcc[OFONO_MAX_MCC_LENGTH + 1];
 	char mnc[OFONO_MAX_MNC_LENGTH + 1];
+	int signal_index; /* If strength is reported via CIND */
+	int signal_min; /* min strength reported via CIND */
+	int signal_max; /* max strength reported via CIND */
+	int tech;
 	unsigned int vendor;
 };
 
@@ -88,6 +90,9 @@ static void at_creg_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		return;
 	}
 
+	if ((status == 1 || status == 5) && (tech == -1))
+		tech = nd->tech;
+
 	cb(&error, status, lac, ci, tech, cbd->data);
 }
 
@@ -103,6 +108,14 @@ static void at_registration_status(struct ofono_netreg *netreg,
 
 	cbd->user = nd;
 
+	/*
+	 * Send *ERINFO to find out the current tech, it will be intercepted
+	 * in mbm_erinfo_notify
+	 */
+	if (nd->vendor == OFONO_VENDOR_MBM)
+		g_at_chat_send(nd->chat, "AT*ERINFO?", none_prefix,
+				NULL, NULL, NULL);
+
 	if (g_at_chat_send(nd->chat, "AT+CREG?", creg_prefix,
 				at_creg_cb, cbd, g_free) > 0)
 		return;
@@ -450,19 +463,6 @@ error:
 	CALLBACK_WITH_FAILURE(cb, data);
 }
 
-static inline void report_signal_strength(struct ofono_netreg *netreg,
-						int strength)
-{
-	DBG("csq_notify: %d", strength);
-
-	if (strength == 99)
-		strength = -1;
-	else
-		strength = (strength * 100) / 31;
-
-	ofono_netreg_strength_notify(netreg, strength);
-}
-
 static void csq_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -477,7 +477,8 @@ static void csq_notify(GAtResult *result, gpointer user_data)
 	if (!g_at_result_iter_next_number(&iter, &strength))
 		return;
 
-	report_signal_strength(netreg, strength);
+	ofono_netreg_strength_notify(netreg,
+				at_util_convert_signal_strength(strength));
 }
 
 static void calypso_csq_notify(GAtResult *result, gpointer user_data)
@@ -494,7 +495,8 @@ static void calypso_csq_notify(GAtResult *result, gpointer user_data)
 	if (!g_at_result_iter_next_number(&iter, &strength))
 		return;
 
-	report_signal_strength(netreg, strength);
+	ofono_netreg_strength_notify(netreg,
+				at_util_convert_signal_strength(strength));
 }
 
 static void option_osigq_notify(GAtResult *result, gpointer user_data)
@@ -511,7 +513,8 @@ static void option_osigq_notify(GAtResult *result, gpointer user_data)
 	if (!g_at_result_iter_next_number(&iter, &strength))
 		return;
 
-	report_signal_strength(netreg, strength);
+	ofono_netreg_strength_notify(netreg,
+				at_util_convert_signal_strength(strength));
 }
 
 static void option_owcti_notify(GAtResult *result, gpointer user_data)
@@ -546,9 +549,10 @@ static void option_octi_notify(GAtResult *result, gpointer user_data)
 	ofono_info("OCTI mode: %d", mode);
 }
 
-static void ste_ciev_notify(GAtResult *result, gpointer user_data)
+static void ciev_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	int strength, ind;
 	GAtResultIter iter;
 
@@ -560,19 +564,22 @@ static void ste_ciev_notify(GAtResult *result, gpointer user_data)
 	if (!g_at_result_iter_next_number(&iter, &ind))
 		return;
 
-	if (ind == SIGNAL_STRENGTH_IND) {
-		if (!g_at_result_iter_next_number(&iter, &strength))
-			return;
+	if (ind != nd->signal_index)
+		return;
 
-		strength = (strength * 100) / 5;
-		ofono_netreg_strength_notify(netreg, strength);
-	}
+	if (!g_at_result_iter_next_number(&iter, &strength))
+		return;
+
+	strength = (strength * 100) / (nd->signal_max - nd->signal_min);
+	ofono_netreg_strength_notify(netreg, strength);
 }
 
-static void ste_cind_cb(gboolean ok, GAtResult *result, gpointer user_data)
+static void cind_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	ofono_netreg_strength_cb_t cb = cbd->cb;
+	struct netreg_data *nd = cbd->user;
+	int index;
 	int strength;
 	GAtResultIter iter;
 	struct ofono_error error;
@@ -591,12 +598,12 @@ static void ste_cind_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		return;
 	}
 
-	/* Skip battery charge level, which is the first reported  */
-	g_at_result_iter_skip_next(&iter);
+	for (index = 1; index < nd->signal_index; index++)
+		g_at_result_iter_skip_next(&iter);
 
 	g_at_result_iter_next_number(&iter, &strength);
 
-	strength = (strength * 100) / 5;
+	strength = (strength * 100) / (nd->signal_max - nd->signal_min);
 
 	cb(&error, strength, cbd->data);
 }
@@ -660,9 +667,15 @@ static void at_signal_strength(struct ofono_netreg *netreg,
 	if (!cbd)
 		goto error;
 
-	if (nd->vendor == OFONO_VENDOR_STE) {
+	cbd->user = nd;
+
+	/*
+	 * If we defaulted to using CIND, then keep using it,
+	 * otherwise fall back to CSQ
+	 */
+	if (nd->signal_index > 0) {
 		if (g_at_chat_send(nd->chat, "AT+CIND?", cind_prefix,
-				ste_cind_cb, cbd, g_free) > 0)
+					cind_cb, cbd, g_free) > 0)
 			return;
 	} else {
 		if (g_at_chat_send(nd->chat, "AT+CSQ", csq_prefix,
@@ -677,6 +690,60 @@ error:
 	CALLBACK_WITH_FAILURE(cb, -1, data);
 }
 
+static void mbm_erinfo_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	GAtResultIter iter;
+	int mode, gsm, umts;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (g_at_result_iter_next(&iter, "*ERINFO:") == FALSE)
+		return;
+
+	if (g_at_result_iter_next_number(&iter, &mode) == FALSE)
+		return;
+
+	if (g_at_result_iter_next_number(&iter, &gsm) == FALSE)
+		return;
+
+	/*
+	 * According to MBM the ERINFO unsolicited response does not contain
+	 * the mode parameter, however at least the MD300 does report it.  So
+	 * we handle both 2 and 3 argument versions
+	 */
+	if (g_at_result_iter_next_number(&iter, &umts) == FALSE) {
+		gsm = mode;
+		umts = gsm;
+	}
+
+	ofono_info("network capability: GSM %d UMTS %d", gsm, umts);
+
+	/* Convert to tech values from 27.007 */
+	switch (gsm) {
+	case 1: /* GSM */
+		nd->tech = 0;
+		break;
+	case 2: /* EDGE */
+		nd->tech = 3;
+		break;
+	default:
+		nd->tech = -1;
+	}
+
+	switch (umts) {
+	case 1: /* UMTS */
+		nd->tech = 2;
+		break;
+	case 2: /* UMTS + HSDPA */
+		nd->tech = 4;
+		break;
+	default:
+		break;
+	}
+}
+
 static void creg_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
@@ -687,11 +754,78 @@ static void creg_notify(GAtResult *result, gpointer user_data)
 				&lac, &ci, &tech, nd->vendor) == FALSE)
 		return;
 
+	if ((status == 1 || status == 5) && tech == -1)
+		tech = nd->tech;
+
 	ofono_netreg_status_notify(netreg, status, lac, ci, tech);
 }
 
-static void at_network_registration_initialized(gboolean ok, GAtResult *result,
-							gpointer user_data)
+static void cind_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	GAtResultIter iter;
+	const char *str;
+	int index;
+	int min, max;
+
+	if (!ok)
+		goto error;
+
+	g_at_result_iter_init(&iter, result);
+	if (!g_at_result_iter_next(&iter, "+CIND:"))
+		goto error;
+
+	index = 1;
+
+	while (g_at_result_iter_open_list(&iter)) {
+		if (!g_at_result_iter_next_string(&iter, &str))
+			goto error;
+
+		if (!g_at_result_iter_open_list(&iter))
+			goto error;
+
+		while (g_at_result_iter_next_range(&iter, &min, &max))
+			;
+
+		if (!g_at_result_iter_close_list(&iter))
+			goto error;
+
+		if (!g_at_result_iter_close_list(&iter))
+			goto error;
+
+		if (g_str_equal("signal", str) == TRUE) {
+			nd->signal_index = index;
+			nd->signal_min = min;
+			nd->signal_max = max;
+		}
+
+		index += 1;
+	}
+
+	if (nd->signal_index == 0)
+		goto error;
+
+	g_at_chat_send(nd->chat, "AT+CMER=3,0,0,1", NULL,
+			NULL, NULL, NULL);
+	g_at_chat_register(nd->chat, "+CIEV:",
+				ciev_notify, FALSE, netreg, NULL);
+	g_at_chat_register(nd->chat, "+CREG:",
+				creg_notify, FALSE, netreg, NULL);
+
+	ofono_netreg_register(netreg);
+	return;
+
+error:
+	ofono_error("This driver is not setup with Signal Strength reporting"
+			" via CIND indications, please write proper netreg"
+			" handling for this device");
+
+	ofono_netreg_remove(netreg);
+}
+
+
+static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
@@ -702,18 +836,16 @@ static void at_network_registration_initialized(gboolean ok, GAtResult *result,
 		return;
 	}
 
-	g_at_chat_register(nd->chat, "+CREG:",
-				creg_notify, FALSE, netreg, NULL);
-	g_at_chat_register(nd->chat, "+CSQ:",
-				csq_notify, FALSE, netreg, NULL);
-
 	switch (nd->vendor) {
+	case OFONO_VENDOR_PHONESIM:
+		g_at_chat_register(nd->chat, "+CSQ:",
+					csq_notify, FALSE, netreg, NULL);
+		break;
 	case OFONO_VENDOR_CALYPSO:
 		g_at_chat_send(nd->chat, "AT%CSQ=1", none_prefix,
 				NULL, NULL, NULL);
 		g_at_chat_register(nd->chat, "%CSQ:", calypso_csq_notify,
 					FALSE, netreg, NULL);
-
 		break;
 	case OFONO_VENDOR_OPTION_HSO:
 		g_at_chat_send(nd->chat, "AT_OSSYS=1", none_prefix,
@@ -754,16 +886,29 @@ static void at_network_registration_initialized(gboolean ok, GAtResult *result,
 		g_at_chat_send(nd->chat, "AT_ODO=0", none_prefix,
 				NULL, NULL, NULL);
 		break;
-	case OFONO_VENDOR_STE:
-		g_at_chat_send(nd->chat, "AT+CMER=3,0,0,1", NULL,
+
+	case OFONO_VENDOR_MBM:
+		g_at_chat_send(nd->chat, "AT*ERINFO=1", none_prefix,
 				NULL, NULL, NULL);
-		g_at_chat_register(nd->chat, "+CIEV:",
-				ste_ciev_notify, FALSE, netreg, NULL);
+		g_at_chat_register(nd->chat, "*ERINFO:", mbm_erinfo_notify,
+					FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT+CIND=?", cind_prefix,
+				cind_support_cb, netreg, NULL);
+		return;
+	case OFONO_VENDOR_HUAWEI:
+		/*
+		 * Huawei doesn't support CIND, signal strength reported
+		 * in the modem driver
+		 */
 		break;
 	default:
-		break;
+		g_at_chat_send(nd->chat, "AT+CIND=?", cind_prefix,
+				cind_support_cb, netreg, NULL);
+		return;
 	}
 
+	g_at_chat_register(nd->chat, "+CREG:",
+				creg_notify, FALSE, netreg, NULL);
 	ofono_netreg_register(netreg);
 }
 
@@ -798,15 +943,13 @@ static void at_creg_test_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 	if (creg2) {
 		g_at_chat_send(nd->chat, "AT+CREG=2", none_prefix,
-				at_network_registration_initialized,
-				netreg, NULL);
+				at_creg_set_cb, netreg, NULL);
 		return;
 	}
 
 	if (creg1) {
 		g_at_chat_send(nd->chat, "AT+CREG=1", none_prefix,
-				at_network_registration_initialized,
-				netreg, NULL);
+				at_creg_set_cb, netreg, NULL);
 		return;
 	}
 
@@ -825,6 +968,7 @@ static int at_netreg_probe(struct ofono_netreg *netreg, unsigned int vendor,
 
 	nd->chat = chat;
 	nd->vendor = vendor;
+	nd->tech = -1;
 	ofono_netreg_set_data(netreg, nd);
 
 	g_at_chat_send(chat, "AT+CREG=?", creg_prefix,
